﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using Microsoft.Web.PlatformInstaller;

namespace ProductCacher
{
    class Program
    {
        protected readonly static string MainXml = "https://go.microsoft.com/fwlink/?LinkId=158722";
        
        static void Main(string[] args)
        {
            ProductManager productManager = new ProductManager();

            // 1. Load main product feeds
            // overload documentation: http://msdn.microsoft.com/en-us/library/ee255981.aspx
            productManager.Load
                (
                    new Uri(MainXml),                    // location of the main feed 
                    false,                               // do not filter products by the current architecture and OS
                    true,                                // load enclosures (eg WebApplicationList.xml)
                    false,                               // no cache yet, so do not use cached directory
                    @"d:\MyCache",                       // where to cache XML feeds (the folder should exist)

                    // get only products available for 
                    //      a. x86 architecture (and those available for both x86 AND x64)
                    //      b. the current OS (major and minor OS versions)
                    //      c. up to service pack 2 (major and minor sevice pack versions)
                    //      d. any product types (e.g. Home, Professional, Ultimate, Web Server, etc)
                    Architecture.x86,                    // arch
                    Environment.OSVersion.Version.Major, // current major version
                    Environment.OSVersion.Version.Minor, // current minor version
                    2,                                   // current major SP
                    0,                                   // current minor SP
                    0  // OsType as defined in this list: http://msdn.microsoft.com/en-us/library/ms724358(VS.85).aspx
                       // 0 if don't want to filter products by OSType
                );

            // 2. Find all the products to cache: .NET Fx 3.5 SP1, SQL Expess, PHP, MySQL, UrlRewrite and SEOToolkit
            Product netFramework35Sp1 = productManager.GetProduct("NETFramework35");
            Product sqlExpress = productManager.GetProduct("SQLExpress");
            Product php = productManager.GetProduct("PHP");
            Product mySql = productManager.GetProduct("MySQL");
            Product SEOtoolkit = productManager.GetProduct("SEOToolkit");
            Product urlRewrite = productManager.GetProduct("UrlRewrite");
            Product webDeploy = productManager.GetProduct("WDeploy");

            // 3. Add all products with the dependencies to the install list
            List<Product> productsToInstall = new List<Product>();
            AddProductWithDependencies(netFramework35Sp1, productsToInstall);
            AddProductWithDependencies(sqlExpress, productsToInstall);
            AddProductWithDependencies(php, productsToInstall);
            AddProductWithDependencies(mySql, productsToInstall);
            AddProductWithDependencies(SEOtoolkit, productsToInstall);
            AddProductWithDependencies(urlRewrite, productsToInstall);
            AddProductWithDependencies(webDeploy, productsToInstall);
            
            // 4. For each product pick the installers (can choose a language of each installer if localized installer is available)
            List<Installer> installersToUse = new List<Installer>();
            Language english = productManager.GetLanguage("en");
            foreach (Product productToInstall in productsToInstall)
            {
                Installer currentInstaller = productToInstall.GetInstaller(english);
                
                // if current product has installer to download (it might be an installer-less Windows component)
                if (currentInstaller != null && currentInstaller.InstallerFile != null)
                {
                    installersToUse.Add(currentInstaller);
                }
            }

            // ***********************************************************************//
            // NOTE: if you use this code, to make WebPI download the installers,
            // WebPI will perform a dependency check against your CURRENT platform.
            // For example, if you are trying to download something incompatible with your
            // current OS, WebPI will throw
            // ***********************************************************************//
            /*
             
            // 5. Prepare installers
            InstallManager installManager = new InstallManager();
            installManager.Load(installersToUse);

            // 6. Download products without installing them
            foreach (InstallerContext installerContext in installManager.InstallerContexts)
            {
                // each installer will be cached under %LocalAppData%\Microsoft\Web Platform Installer\installers\<product Id>\<installer's SHA1>\
                string failureReason;
                installManager.DownloadInstallerFile(installerContext, out failureReason);
                if (!string.IsNullOrEmpty(failureReason))
                {
                    Console.Error.WriteLine("Web PI download failure: " + failureReason);
                }
                    
            }
            // ***********************************************************************/

            // 5. Download the installers in the same folder Web PI does, aka Installer Cache
            string WebPIinstallersFolder = Environment.ExpandEnvironmentVariables(@"%LocalAppData%\Microsoft\Web Platform Installer\installers");
            if (!Directory.Exists(WebPIinstallersFolder)) 
            {
                Directory.CreateDirectory(WebPIinstallersFolder);
            }
            foreach (Installer installer in  installersToUse) 
            {
                // re-create WebPI behavior: download each installer 
                // under %LocalAppData%\Microsoft\Web Platform Installer\installers\<product Id>\<installer's SHA1>\
                string currentInstallerDirectory = 
                    Path.Combine(WebPIinstallersFolder, installer.Product.ProductId) + 
                    "\\" + installer.InstallerFile.SHA1Hash;
                
                // if the directory <productId>\<installer's hash> doesn't exist, create it
                if (!Directory.Exists(currentInstallerDirectory)) 
                {
                    Directory.CreateDirectory(currentInstallerDirectory);
                }    
            
                WebClient downloadClient = new WebClient();
                downloadClient.DownloadFile
                    (
                        installer.InstallerFile.InstallerUrl,
                        Path.Combine(currentInstallerDirectory, installer.Product.ProductId + 
                        // NOTE: by far not the best method to get a file extension, 
                        // consider the case where your absolute URL does not contain file extension
                        Path.GetExtension(installer.InstallerFile.InstallerUrl.AbsolutePath))
                    );
            }
        }

        private static void AddProductWithDependencies(Product product, List<Product> productsToInstall)
        {
            // avoid duplicates 
            if (!productsToInstall.Contains(product))
            {
                productsToInstall.Add(product);
            }

            // GetMissingDependencies will check whether any dependencies are not installed for the product
            // AND are not already added to the install list (productsToInstall)
            ICollection<Product> missingDependencies = product.GetMissingDependencies(productsToInstall);
            if (missingDependencies != null)
            {
                foreach (Product dependency in missingDependencies)
                {
                    AddProductWithDependencies(dependency, productsToInstall);
                }
            }
        }
    }
}
